# -*- coding: utf-8 -*-
import time
from main.GUIA import execute, GControl, GOperator
from main.OperationSet import *
from functools import wraps
import uiautomation as uia
from main.GAutoit import execute as gai


if __name__ == '__main__':
    # pass
    # execute(calculator)
    # execute(calculator1)
    # gai(autoit_notepad)
    # execute(youdao_dict)
    # execute(youdao_dict1)
    # execute(word)
    # *******************
    # execute(postal_login)
    # time.sleep(20)
    # execute(postal_login_btn)
    # execute(postal_search)
    # execute(postal_save)
    # *******************
    # execute(icbc_ele_search)
    execute(icbc_ele_save)